require 'thor'
require 'touchgfx/version'

module TouchGFX
  module CLI
    class Base < Thor
      include Thor::Actions

      def self.touchgfx_root
        unless @touchgfx_env
          @touchgfx_env = ENV['touchgfx']
          unless @touchgfx_env
            puts 'No touchgfx environment variable...'
            path = Dir.pwd
            level = 0
            until File.directory?(File.join(path, '/', 'touchgfx', 'framework')) do
              level += 1
              fail if level > 10
              path=File.dirname(path)
            end
            @touchgfx_env = '../'*level #WAS: path
            ENV['touchgfx'] = @touchgfx_env
          end
          if @touchgfx_env[-1] == '/'
            @touchgfx_env = @touchgfx_env[0..-2]
            ENV['touchgfx'] = @touchgfx_env
          end
          puts "Using touchgfx #{@touchgfx_env}"
        end
        @touchgfx_env
      end

      def self.source_root
        File.expand_path(File.join(touchgfx_root,'app','template','EmptyApplication'))
      end

      desc "list COMPILER", "list configurations"
      def list(compiler)
        confs = Dir[File.join("#{Base.source_root}","target","*/*", "#{compiler}")]
        puts confs.each_with_index.map {|e,i| "#{i+1}) #{e}" }
      end

      desc "version", "version information"
      def version
        puts "touchgfx cli, version #{TouchGFX::VERSION}"
      end
    end
  end
end
