require 'touchgfx/cli/base'

#TODO require these elsewhere - this method is deprecated
require 'touchgfx/generators/gcc'
require 'touchgfx/generators/msvs'
require 'touchgfx/generators/iar'
require 'touchgfx/generators/keil'

module TouchGFX
  module CLI
    class Generate < Base
      option :c, :required => true, :banner => "<company>"
      option :b, :required => true, :banner => "<board>"
      option :clean, :type => :boolean, :default => false
      desc "iar", "create iar workspace and project files for this application."
      long_desc <<-DESCRIPTION
      Create iar .eww and .ewp files

      Update the .ewp file with all source files found in the application

      With the --clean option, the .ewp file will have all existing application source files removed prior to the creation

      DESCRIPTION
      def iar
        company = options[:c]
        board = options[:b]
        if iar_support?(company, board)
          Generators::IAR.source_root(Base.source_root)
          Generators::IAR.start([company, board, options[:clean]])
        else
          puts "Supported boards:"
          puts iar_boards.map { |c,b| "#{c} #{b}" }.join("\n")
        end
      end

      desc "makefile", "create Makefile for this application"
      def makefile
        copy_file 'Makefile', :skip => true
      end

      #Don't really need these anymore
      private

      def iar_support?(company, board)
        iar_boards.find { |iar_board| iar_board == [company, board]}
      end

      def iar_boards
        Dir[File.join(Base.source_root, '**', 'IAR')].collect do |iar_directory|
          match = iar_directory.match(/target\/(.*)\/(.*)\/IAR/)
          match ? [match[1], match[2]] : nil
        end.compact
      end
    end
  end
end
