require 'touchgfx/cli/base'
require 'touchgfx/cli/generate'
require 'touchgfx/generators/update'
require 'touchgfx/generators/update_project'

module TouchGFX
  module CLI
    class Inside < Base

      #desc "generate [name]", "generate or update bits and pieces"
      #subcommand 'generate', Generate

      #TODO: Share compiler options between inside and outside update methods - Maybe revert to Thor
      method_option :compiler, :type => :string, :aliases => "-c", :desc => "Specifies compiler", :default => "iar;mingw32;xgcc;msvs;keil" #Default all compilers
      method_option :clean, :type => :boolean, :aliases => "--clean", :desc => "Clean", :optional => true, :default => false
      method_option :bpp, :type => :numeric, :aliases => "--use-bpp", :desc => "Bit depth of project", :optional => true, :default => 0
      method_option :nonopaque, :type => :string, :aliases => "--non-opaque", :desc => "16bpp non-opaque format rgb565 (default) or argb8888", :optional => true, :default => ''
      method_option :bwrle, :type => :string, :aliases => "--bw-rle", :desc => "Use BW_RLE over BW for 1bpp", :optional => true, :default => ''
      method_option 'iar-project', :type => :string, :desc => "Specify which .ewp to update", :optional => true, :default => ''
      method_option 'keil-project', :type => :string, :desc => "Specify which .uvproj{x} to update", :optional => true, :default => ''

      desc "update", "update this application"
      def update
        puts "\n======================================================================="
        puts "Processing #{Dir.pwd}"
        puts "======================================================================="

        if ![0,1,2,4,16,18,24].include?(options[:bpp])
          puts "Unsupported --use-bpp=#{options[:bpp]} specified, please use only 1, 2, 4, 16, 18 or 24"
        elsif !['','rgb565','argb8888'].include?(options[:nonopaque])
          puts "Unsupported --nonopaque=#{options[:nonopaque]} specified, please use only rgb565 or argb8888"
        elsif !['','yes','no'].include?(options[:bwrle])
          puts "Unsupported --bw-rle=#{options[:bwrle]} specified, please use only yes or no"
        else
          Generators::Base.source_root(Base.source_root)
          Generators::Update.new(Generators::Base.new, options).run
        end
      end

      method_option 'project-file', :type => :string, :desc => "Specify which project file (.ewp, .uvproj{x}) to update", :required => true
      method_option 'gui-group-name', :type => :string, :desc => "Name of group holding gui files", :default => 'gui'
      method_option 'generated-group-name', :type => :string, :desc => "Name of group holding generated files", :default => 'generated'
      desc "update_project", "update project"
      def update_project
        Generators::UpdateProject.new(options['project-file'],options['gui-group-name'],options['generated-group-name']).run
      end

    end
  end
end
