require 'touchgfx/cli/base'
require 'touchgfx/generators/project'

#We should also be able to update from outside - recursively.

module TouchGFX
  module CLI
    class Outside < Base

      desc "new NAME", "create new TouchGFX project"
      def new(name)
        Generators::Project.source_root(Base.source_root)
        Generators::Project.start([name])
      end

      # TODO: Share compiler options between inside and outside update methods
      # http://stackoverflow.com/questions/14346285/how-to-make-two-thor-tasks-share-options
      method_option :compiler, :type => :string, :aliases => "-c", :desc => "Specifies compiler", :default => "iar;mingw32;xgcc;msvs;keil"
      method_option :target, :aliases => "-t", :desc => "Specifies target" #default?
      method_option :clean, :type => :boolean, :aliases => "--clean", :desc => "Clean", :optional => true, :default => false
      method_option :bpp, :type => :numeric, :aliases => "--use-bpp", :desc => "Bit depth of project", :option => true, :default => 0
      method_option :nonopaque, :type => :string, :aliases => "--non-opaque", :desc => "16bpp non-opaque format rgb565 (default) or argb8888", :optional => true, :default => ''
      method_option :bwrle, :type => :boolean, :aliases => "--bw-rle", :desc => "Use BW_RLE over BW for 1bpp", :optional => true, :default => false
      method_option 'iar-project', :type => :string, :desc => "Specify which .ewp to update", :optional => true, :default => ''
      method_option 'keil-project', :type => :string, :desc => "Specify which .uvproj{x} to update", :optional => true, :default => ''

      desc "update", "Update applications recursively"
      def update
        pwd = Dir.pwd+'/'
        if File.exist?('framework/source/touchgfx/Application.cpp')
          puts "Updating TouchGFX Core"
          root_dir = pwd.gsub(/^(.*)\/touchgfx\/.*/,'\1')
          Dir.chdir(root_dir) do
             Dir['touchgfx/build_conf/core/**/TouchGFX.vcxproj'].each do |vcxproj|
              puts "\n======================================================================="
              puts "Updating #{vcxproj}"
              puts "======================================================================="
              Generators::MSVS.source_root(Base.source_root)
              Generators::MSVS.start([vcxproj, options[:clean]])
            end
            Dir['touchgfx/build_conf/core/**/touchgfx.uvproj*'].each do |uvproj|
              puts "\n======================================================================="
              puts "Updating #{uvproj}"
              puts "======================================================================="
              Generators::Keil.source_root(Base.source_root)
              Generators::Keil.start([uvproj, options[:clean]])
            end
            Dir['touchgfx/build_conf/core/**/touchgfx.ewp'].each do |ewpfile|
              puts "\n======================================================================="
              puts "Updating #{ewpfile}"
              puts "======================================================================="
              Generators::IAR.source_root(Base.source_root)
              Generators::IAR.start([ewpfile, options[:clean]])
            end
          end
        end

      end

    end
  end
end
