require 'touchgfx/generators/base'
require 'nokogiri'

module TouchGFX
  module Generators
    class GCC < Base

      argument :project_file
      argument :bpp, :optional => true
      argument :nonopaque, :optional => true
      argument :bwrle, :optional => true
      PROJ_DIR=''

      def check_project_node
#        if project_file_xml.at_xpath('./project').nil?
#          say_staus :conflict, "no <project> node in #{project_file}", :red # #{project_file}
#          Kernel.abort
#        end
      end

      def project_file_contents
        @tgfx_path = "#{ENV['touchgfx'].strip}/touchgfx"
        # Force setup, not a nice way to do this
        project_file_xml

        find_sdl_version

        patch_makefile

        update_properties
        output_project_file
      end

      private

      def find_sdl_version
        @sdl_version=1
        @subsystem=0 #main
        if File.exist?('simulator/main.cpp')
          contents = File.read('simulator/main.cpp', :encoding=>'utf-8')
          if contents.match(/HALSDL2/)
            @sdl_version=2
          end
          if contents.match(/int CALLBACK WinMain/)
            @subsystem=1 #windows
          end
        end
      end

      def patch_makefile
        contents = File.read(project_file, :encoding=>'utf-8')

        # Ignore errors from includes
        contents.gsub!(/^(include )/,
                       "-\\1")
        # Allow absolute paths for touchgfx_path
        contents.gsub!('$(application_path)/$(touchgfx_path)',
                       '$(touchgfx_path)')
        # Remove Darwin sections
        contents.gsub!(/^ifeq \(\$\(UNAME\), Darwin\)\n(?:.*\n)*?else\n((?:.*\n)*?)endif/,
                       "\\1")
        # Change framework_source to not include platform/driver/touch
        contents.gsub!(/^\t\$\(touchgfx_path\)\/framework\/source\/platform\/driver\/touch \\\n/,
                       '')
        # Now remove old definition of framework_files
        contents.gsub!(/^framework_files := .*\n/,
                       '')
        # Now update framework_source to appropriate SDL
        contents.gsub!(/(\$\(touchgfx_path\)\/framework\/source\/platform\/hal\/simulator\/)sdl2?/,
                       "\\1#{@sdl_version==1?'sdl':'sdl2'}\\2")
        # Now add framework_files to appropriate SDL
        contents.gsub!(/^(framework_source :=.*)$/,
                       "framework_files := \$(touchgfx_path)/framework/source/platform/driver/touch/SDL#{@sdl_version==1?'':'2'}TouchController.cpp\n\\1")
        # Now make sure framework_files is part of source_files
        contents.gsub!(/^(source_files := .*?BitmapDatabase.cpp).*$/,
                       "\\1 $(framework_files)")
        # Replace the way object_files are constructed
        contents.gsub!("object_files := \$(source_files:%.cpp=\$(object_output_path)/%.o) \$(c_source_files:%.c=\$(object_output_path)/%.o)\nobject_files := \$(patsubst \$(object_output_path)/\$(touchgfx_path)/%,\$(object_output_path)/touchgfx/%,\$(object_files))",
                       "object_files := \$\(source_files:\$\(touchgfx_path\)/%.cpp=\$\(object_output_path\)/touchgfx/%.o\) \$\(c_source_files:\$\(touchgfx_path\)/%.c=\$\(object_output_path\)/touchgfx/%.o\)\nobject_files := \$\(object_files:%.cpp=\$\(object_output_path\)/%.o\)\nobject_files := \$\(object_files:%.c=\$\(object_output_path\)/%.o\)")
        # Use framework_files in source_files
        contents.gsub!(/^source_files := .*$/,
                       'source_files := $(call find, $(source_paths),*.cpp) $(framework_files)')

        if @sdl_version==1
          # Change libraries/includes for linux:
          contents.gsub!(/^(library_path.*?\/)sdl2(\/linux64.*?\n)libraries :=(.*?) SDL2 SDL2_image rt (.*?)$/,
                         "\\1sdl\\2libraries :=\\3 SDL SDL_image SDLmain caca webp \\4")
          contents.gsub!(/^(library_includes \+= .*?\/)sdl2\/vendor(\nlinker_options_local := .*?\/lib\/)sdl2(\/linux64\/\))$/,
                         "\\1sdl/vendor/linux64\\2sdl\\3")

          # Change libraries/includes for mingw32
          contents.gsub!(/(^sdl_library_path := .*?\/lib\/)sdl2(\/win32\nlibrary_path := .*?\nlibraries :=.*?) SDL2 SDL2_image (.*$)/,
                         "\\1sdl\\2 SDL SDL_image SDLmain \\3")
          contents.gsub!(/^(library_includes \+= .*?\/)sdl2\/vendor(.*?)(\nresource_file)/,
                         "\\1sdl/vendor/win32\\2\\3")

          # Linux
          contents.gsub!(/\t@if \[ -f \$\(application_path\)\/simulator\/landscape.png \]; then cp \$\(application_path\)\/simulator\/landscape.png \$\(binary_output_path\); fi\n\t@if \[ -f \$\(application_path\)\/simulator\/portrait.png \]; then cp \$\(application_path\)\/simulator\/portrait.png \$\(binary_output_path\); fi\n(\t@echo .To be able to run.*\n.*)sdl2/,
                         "\\1sdl")
          # Windows
          contents.gsub!(/\t@cp \$\(sdl_library_path\)\/SDL2.dll \$\(binary_output_path\)\n\t@cp \$\(sdl_library_path\)\/SDL2_image.dll \$\(binary_output_path\)\n\t@cp \$\(sdl_library_path\)\/libpng16-16.dll \$\(binary_output_path\)\n\t@cp \$\(sdl_library_path\)\/zlib1.dll \$\(binary_output_path\)\n\t@if \[ -f \$\(application_path\)\/simulator\/landscape.png \]; then cp \$\(application_path\)\/simulator\/landscape.png \$\(binary_output_path\); fi\n\t@if \[ -f \$\(application_path\)\/simulator\/portrait.png \]; then cp \$\(application_path\)\/simulator\/portrait.png \$\(binary_output_path\); fi\n/,
                         "\t@cp $(sdl_library_path)/SDL.dll $(binary_output_path)\n\t@cp $(sdl_library_path)/SDL_image.dll $(binary_output_path)\n")
        else
          # Change libraries/includes for linux:
          contents.gsub!(/^(library_path.*?\/)sdl(\/linux64.*?\n)libraries :=(.*?) SDL SDL_image SDLmain caca webp (.*?)$/,
                         "\\1sdl2\\2libraries :=\\3 SDL2 SDL2_image rt \\4")
          contents.gsub!(/^(library_includes \+= .*?\/)sdl\/vendor\/linux64(\nlinker_options_local := .*?\/lib\/)sdl(\/linux64\/\))$/,
                         "\\1sdl2/vendor\\2sdl2\\3")

          # Change libraries/includes for mingw32
          contents.gsub!(/(^sdl_library_path := .*?\/lib\/)sdl(\/win32\nlibrary_path := .*?\nlibraries := .*?) SDL SDL_image SDLmain (.*$)/,
                         "\\1sdl2\\2 SDL2 SDL2_image \\3")
          contents.gsub!(/(^library_includes \+= .*?\/)sdl\/vendor\/win32(.*?)(\nresource_file)/,
                         "\\1sdl2/vendor\\2\\3")

          # Linux
          contents.gsub!(/^(\t@echo .To be able to run.*?\n.*?\/)sdl(\/.*?\n)/,
                         "\t@if [ -f $(application_path)/simulator/landscape.png ]; then cp $(application_path)/simulator/landscape.png $(binary_output_path); fi\n\t@if [ -f $(application_path)/simulator/portrait.png ]; then cp $(application_path)/simulator/portrait.png $(binary_output_path); fi\n\\1sdl2\\2")
          # Windows
          contents.gsub!(/\t@cp \$\(sdl_library_path\)\/SDL.dll \$\(binary_output_path\)\n\t@cp \$\(sdl_library_path\)\/SDL_image.dll \$\(binary_output_path\)\n/,
                         "\t@cp $(sdl_library_path)/SDL2.dll $(binary_output_path)\n\t@cp $(sdl_library_path)/SDL2_image.dll $(binary_output_path)\n\t@cp $(sdl_library_path)/libpng16-16.dll $(binary_output_path)\n\t@cp $(sdl_library_path)/zlib1.dll $(binary_output_path)\n\t@if [ -f $(application_path)/simulator/landscape.png ]; then cp $(application_path)/simulator/landscape.png $(binary_output_path); fi\n\t@if [ -f $(application_path)/simulator/portrait.png ]; then cp $(application_path)/simulator/portrait.png $(binary_output_path); fi\n")
        end

        # Remove libextra
        contents.gsub!(/^libextra.*\n/,'')
        # Insert proper libextra
        linux=true
        contents.gsub!(/^(libend.*\n)/) do |libend|
          if linux
            linux=false
            "#{libend}libextra :=\n"
          elsif @subsystem==0
            "#{libend}libextra :=\n"
          else
            "#{libend}libextra := -Wl,--subsystem,windows\n"
          end
        end
        # Add $(libextra)
        contents.gsub!(/(\$\(libend\)).*$/,"\\1 $(libextra)")

#        # Now fixup libraries and include paths
#        # ... first Linux
#        srch = contents.match(/(^ifeq \(\$\(UNAME\), Linux\).*?else)/m)[0]
#        rplc = srch.gsub(/^(libraries :=.*?)SDL.*$/,
#                         "\\1#{@sdl_version==1?'SDL SDL_image SDLmain caca webp':'SDL2 SDL2_image rt'} m pthread dl")
#        rplc.gsub!(/^(library_includes \+= .*?)sdl.*$/,
#                  "\\1#{@sdl_version==1?'sdl/vendor/linux64':'sdl2/vendor'}")
#        rplc.gsub!(/^(linker_options_local := .*sdl).*?(\/linux64\))$/,
#                  "\\1#{@sdl_version==1?'':'2'}\\2")
#        contents.gsub!(srch,rplc)
#        # ... next Windows
#        srch = contents.match(/(^else\n-include \$\(touchgfx_path\)\/config\/toolchain-mingw32.mk.*?endif)/m)[0]
#        puts srch
#        rplc = srch.gsub(/^(libraries :=.*?)SDL.*$/,
#                         "\\1#{@sdl_version==1?'SDL SDL_image SDLmain':'SDL2 SDL2_image'} m pthread mingw32")
#        rplc.gsub!(/^(library_includes \+= .*?)sdl.*$/,
#                  "\\1#{@sdl_version==1?'sdl/vendor/win32':'sdl2/vendor'}")
#        contents.gsub!(srch,rplc)
#        # Finally the copying of files
#        # ... first Linux
#        # ... next Windows

        write_file(project_file, contents)
      end

      def update_properties
        if File.exist?(@properties_file)
          # Update TouchGFX environment path:
          tgfx_path = @properties_file_content.match(/^(touchgfx_path *:= *).*$/)
          if tgfx_path
            @properties_file_content.gsub!(/^#{tgfx_path[0]}/,"#{tgfx_path[1]}#{tgfx_rel_path.gsub('\\','/')}")
          end

          # Update BPP
          bpp_match = @properties_file_content.match(/^(user_cflags *:=.*-DUSE_BPP=)(\d+)/)
          new_bpp = bpp == 18 ? 24 : bpp == 0 ? 16 : bpp
          if !bpp_match
            # No -DUSE_BPP found, add one
            user_cflags_match = @properties_file_content.match(/^user_cflags *:=.*$/)
            if user_cflags_match
              @properties_file_content.gsub!(user_cflags_match[0],"#{user_cflags_match[0]} -DUSE_BPP=16")
            else
              @properties_file_content = "#{@properties_file_content}\nuser_cflags := -DUSE_BPP=16\n"
            end
            bpp_match = @properties_file_content.match(/^(user_cflags *:=.*-DUSE_BPP=)\d+/)
          else
            if bpp == 0
              new_bpp = bpp_match[2].to_i
            end
          end
          fail unless bpp_match
          @properties_file_content.gsub!(/^#{bpp_match[0]}/, "#{bpp_match[1]}#{new_bpp}")

          # Update opaque_image_format
          opaque_match = @properties_file_content.match(/^(opaque_image_format *:= *)(.*)$/)
          unless opaque_match
            @properties_file_content = "#{@properties_file_content}\nopaque_image_format := RGB565\n"
            opaque_match = @properties_file_content.match(/^(opaque_image_format *:= *)(.*)$/)
          end
          opaque_image_format = opaque_match[2]
          case new_bpp
          when 1
            if !['BW_RLE','BW'].include?(opaque_image_format)
              opaque_image_format = 'BW'
            end
            if bwrle != ''
              opaque_image_format = bwrle == 'yes' ? 'BW_RLE' : 'BW'
            end
          when 2
            opaque_image_format = 'GRAY2'
          when 4
            opaque_image_format = 'GRAY4'
          when 16
            opaque_image_format = 'RGB565'
          when 18
            opaque_image_format = 'RGB666'
          when 24
            opaque_image_format = 'RGB888'
          end
          @properties_file_content.gsub!(/^#{opaque_match[0]}/, "#{opaque_match[1]}#{opaque_image_format}")

          # Update non_opaque_image_format
          non_opaque_match = @properties_file_content.match(/^(non_opaque_image_format *:= *)(.*)$/)
          unless non_opaque_match
            @properties_file_content = "#{@properties_file_content}\nnon_opaque_image_format := RGB565\n"
            non_opaque_match = @properties_file_content.match(/^(non_opaque_image_format *:= *)(.*)$/)
          end
          non_opaque_image_format = non_opaque_match[2]
          case new_bpp
          when 1
            if !['BW_RLE','BW'].include?(non_opaque_image_format)
              non_opaque_image_format = 'BW'
            end
            if bwrle != ''
              non_opaque_image_format = bwrle == 'yes' ? 'BW_RLE' : 'BW'
            end
          when 2
            non_opaque_image_format = 'GRAY2'
          when 4
            non_opaque_image_format = 'GRAY4'
          when 16
            old_format = non_opaque_match[2]
            if !['RGB565','ARGB8888'].include?(old_format)
              old_format = 'RGB565'
            end
            non_opaque_image_format = nonopaque == 'rgb565' ? 'RGB565' : nonopaque == 'argb8888' ? 'ARGB8888' : old_format
          when 18
            non_opaque_image_format = 'ARGB8888'
          when 24
            non_opaque_image_format = 'ARGB8888'
          end
          @properties_file_content.gsub!(/^#{non_opaque_match[0]}/, "#{non_opaque_match[1]}#{non_opaque_image_format}")

        end
      end

      def target
        File.join(Dir.pwd)
      end

      def tgfx_rel_path
        unless @tgfx_rel_path
          tgfx_array = @tgfx_path.gsub(/^(.:)/){$1.downcase}.gsub('\\','/').split('/')
          cur_array = Dir.pwd.gsub(/^(.:)/){$1.downcase}.gsub('\\','/').split('/')
          while tgfx_array[0] == cur_array[0]
            tgfx_array = tgfx_array[1..-1]
            cur_array = cur_array[1..-1]
          end
          @tgfx_rel_path = File.join(['..']*cur_array.size, tgfx_array)
          if PROJ_DIR.length > 0
            @tgfx_rel_path = File.join(PROJ_DIR, @tgfx_rel_path)
          end
          if @tgfx_rel_path.match(/:/) || @tgfx_path.start_with?('..')
            @tgfx_rel_path = @tgfx_path
          end
          @tgfx_rel_path.gsub!('/','\\')
        end
        return @tgfx_rel_path
      end

      def write_file(file, output, force=false)
        input = File.read(file, :encoding=>'utf-8')
        if force || input != output
          File.open(file, 'w:utf-8') { |f| f.write(output) }
          return true
        end
        return false
      end

      def project_file_xml
        @properties_file = "config/gcc/app.mk"
        @properties_file_content = IO.read(@properties_file)
#        @project_file_xml ||= Nokogiri::XML(File.read(project_file, :encoding=>'utf-8')) do |config|
#          config.default_xml.noblanks
#        end
      end

      def output_project_file
        write_file(@properties_file, @properties_file_content)
#        File.open(project_file, 'w:utf-8') { |f| f.write(project_file_xml.to_xml(:indent => 2)) }
      end
    end
  end
end
