module TouchGFX
  module Generators
    #first tried: http://stackoverflow.com/questions/17332255/how-do-i-use-thor-actions-without-requiring-a-thor-cli-app
    #which didn't work
    #then tried this :)

    # basic ruby class, pass in the preconfigured thor-helper-generator
    # TODO move this into its own file
    class BaseGenerator
      attr_reader :generator
      attr_reader :options
      def initialize(generator, options)
        @generator = generator
        @options = options
      end
    end

    #TODO: Remove the need for a compiler flag. Just do
    # everything the application asks (adhering to naming of
    # target folders). In this way, we can update a bulk of
    # applications that have different compiler/target
    # support. => This requires an update to tell the user
    # whether or not his target folder configuration is invalid.


    class Update < BaseGenerator

      def par
        @par ||= OS.linux? ?
                   "-j#{system_processor_count}" :
                   `make -v`.match(/^GNU Make 4\.1$/) ?
                     "-j#{system_processor_count}" :
                     ''
      end

      def run
        subdirs = []

        if options[:clean]
          Dir['generated/*'].reject{|d|d.match('/gui_generated$')}.each do |d|
            FileUtils.rm_rf(d)
          end
        end

        if options[:compiler].include? "mingw32"
          # ------------------
          # GCC specific stuff
          # ------------------
          #run gcc generator for each project file
          project_files = [ File.join("simulator","gcc") ]
          #Call specific updater for each project file.
          #Copy target configuration from source root
          project_files.each do |d|
            if Dir.exist?(d)
              if options[:clean]
                unless Dir.pwd == Base.source_root
                  generator.remove_dir d, :verbose => false
                  generator.empty_directory d, :verbose => false
                end
                generator.directory d, :verbose => false, :recursive => false, :force => options[:force]
              end
              Dir[File.join(d,"Makefile*")].each do |file|
                puts "(MINGW32) Updating '#{file}'"
                gcc(file)
              end
              puts "(MINGW32) Generating assets."
              # Makefile has been updated, regenerate assets
              result = system("make -f #{d}/Makefile assets -r #{par}")
              subdirs.push(d.split('/')[0...-1].join('/'))
            end
          end
        end

        if options[:compiler].include? "msvs"
          # -------------------
          # MSVS specific stuff
          # -------------------
          #run gcc generator for each project file
          msvs_files = [ File.join("simulator","msvs") ]
          #Call specific updater for each project file.
          #Copy target configuration from source root
          msvs_files.each do |d|
            if Dir.exist?(d)
              if options[:clean]
                unless Dir.pwd == Base.source_root
                  generator.remove_dir d, :verbose => false
                  generator.empty_directory d, :verbose => false
                end
                generator.directory d, :verbose => false, :recursive => false, :force => options[:force], :exclude_pattern => /\.(opensdf|suo|sdf)$/
              end
              Dir[File.join(d,"*.vcxproj")].each do |file|
                puts "(MSVS) Updating '#{file}'"
                msvs(file)
              end
              subdirs.push(d.split('/')[0...-1].join('/'))
            end
          end
        end

        #This is a standard way of doing this, make it common in some way.
        if options[:compiler].include? "keil"
          # -------------------
          # Keil specific
          # -------------------
          if options['keil-project'] && !options['keil-project'].empty?
            project_files = [ options['keil-project'] ]
          else
            targets_str = File.join("target","**","Keil")
            targets = Dir["#{targets_str}"].reject{|f|f.match(/target\/Keil/)}
            if options[:clean]
              targets.each do |d|
                unless Dir.pwd == Base.source_root
                  generator.remove_dir d, :verbose => false
                  generator.empty_directory d, :verbose => false
                end
                generator.directory "#{d}", :verbose => false, :recursive => false, :force => options[:force]
              end
            end

            #run Keil generator for each project file
            project_files = Dir[File.join("#{targets_str}","*.uvproj"),
                                File.join("#{targets_str}","*.uvprojx")]
          end

          #Call specific updater for each project file.
          project_files.each do |f|
            puts "(Keil) Updating '#{f}' with assets in 'generated'"
            keil(f)
            subdirs.push(f.split('/')[0...-2].join('/'))
          end
        end

        if options[:compiler].include? "iar"
          # -------------------
          # IAR specific
          # -------------------

          if options['iar-project'] && !options['iar-project'].empty?
            project_files = [ options['iar-project'] ]
          else
            targets_str = File.join("target","**","{IAR,IAR8.x}")
            targets = Dir["#{targets_str}"]

            if options[:clean]
              #Copy target configuration from source root
              targets.each do |d|
                unless Dir.pwd == Base.source_root
                  generator.remove_dir d, :verbose => false
                  generator.empty_directory d, :verbose => false
                end
                generator.directory d, :verbose => false, :recursive => false, :force => options[:force]
              end
            end

            #TODO: Remove the need for a compiler flag. Just do
            # everything the application asks (adhering to naming of
            # target folders). In this way, we can update a bulk of
            # applications that have different compiler/target
            # support. => This requires an update to tell the user
            # whether or not his target folder configuration is invalid.

            #---------------------------------------------
            # IAR specific stuff - Should we seperate this in subcommands?
            # ---------------------------------------------
            #run iar generator for each project file
            project_files = Dir[File.join("#{targets_str}","*.ewp")]
            #puts Dir.pwd
          end

          #Call specific updater for each project file.
          project_files.each do |f|
            puts "(IAR) Updating '#{f}' with assets in 'generated'"
            iar(f) # Choose the proper compiler here. We may need to
                   # have compilers as subcommands if we want to make
                   # it nice to extend.
          end

          targets.each{|t| subdirs.push(t.split('/')[0...-1].join('/')) }
        end

        if options[:compiler].include? "xgcc"
          if options[:clean]
            targets = Dir[File.join("target","**","gcc")]
            targets.each do |d|
              puts "(XGCC) Updating '#{d}'"
              unless Dir.pwd == Base.source_root
                generator.remove_dir d, :verbose => false
                generator.empty_directory d, :verbose => false
              end
              generator.directory d, :verbose => false, :recursive => false, :force => options[:force]
            end
          end
          targets.each{|t| subdirs.push(t.split('/')[0...-1].join('/')) }
        end

        if options[:clean]
          scandirs = []
          subdirs.uniq.each do |dir|
            while !dir.empty?
              scandirs.push(dir)
              dir = dir.split('/')[0...-1].join('/')
            end
          end
          scandirs.uniq.each do |dir|
            Dir[File.join(Base.source_root, dir, '*')].select{|f|File.file?(f)}.reject{|f| /^.*\/main\.cpp$/.match(f)}.each do |f|
              filename = f.split('/')[-1]
              unless File.join(Dir.pwd, dir, filename) == f
                puts "(COPY) Updating '#{dir}/#{filename}' from #{f}"
                FileUtils.cp(f, dir)
              end
            end
          end
        end
      end

      private

      #IAR updater  - Maybe just redefine Update class in another file
      def iar(project_file)
        Generators::IAR.source_root(Base.source_root)
        Generators::IAR.start([project_file, options[:bpp], options['iar-project'] && !options['iar-project'].empty?])
      end

      #Keil updater  - Maybe just redefine Update class in another file
      def keil(project_file)
        Generators::Keil.source_root(Base.source_root)
        Generators::Keil.start([project_file, options[:bpp], options['keil-project'] && !options['keil-project'].empty?])
      end

      #GCC Makefile updater
      def gcc(project_file)
        Generators::GCC.source_root(Base.source_root)
        Generators::GCC.start([project_file, options[:bpp], options[:nonopaque], options[:bwrle]])
      end

      #MSVS Makefile updater
      def msvs(project_file)
        Generators::MSVS.source_root(Base.source_root)
        Generators::MSVS.start([project_file, options[:bpp], options[:nonopaque], options[:bwrle]])
      end
    end
  end
end

module OS
  def OS.windows?
    (/cygwin|mswin|mingw|bccwin|wince|emx/ =~ RUBY_PLATFORM) != nil
  end

  def OS.mac?
   (/darwin/ =~ RUBY_PLATFORM) != nil
  end

  def OS.unix?
    !OS.windows?
  end

  def OS.linux?
    OS.unix? and not OS.mac?
  end
end

def system_processor_count
  require 'parallel'
  Parallel.processor_count || 1
end
