require 'touchgfx/generators/iar_project_updater'
require 'touchgfx/generators/keil_project_updater'
require 'touchgfx/generators/msvs_project_updater'
require 'touchgfx/generators/sw4_project_updater'
require 'touchgfx/generators/cubeide_project_updater'

module TouchGFX
  module Generators
    class UpdateProject
      attr_reader :project_file
      attr_reader :touchgfx_path
      attr_reader :touchgfx_env

      def initialize(project_file, gui_group_name, generated_group_name)
        @project_file = project_file
        @gui_group_name = gui_group_name
        @generated_group_name = generated_group_name
      end

      def run
        app_mk = File.read('config/gcc/app.mk')
        config = Struct.new(:bpp, :opaque, :nonopaque).new
        @touchgfx_path = app_mk[/touchgfx_path := (.*)$/,1]
        @touchgfx_env = app_mk[/touchgfx_env := (.*)$/,1]
        config.bpp = app_mk[/-DUSE_BPP=(\d+)/,1]
        config.opaque = app_mk[/opaque_image_format\s*:=\s*(\w+)/,1]
        config.nonopaque = app_mk[/non_opaque_image_format\s*:=\s*(\w+)/,1]

        generator = case project_file
        when /\.ewp$/
          Generators::IARProjectUpdater
        when /\.ipcf$/
          Generators::IARProjectConnector
        when /.uvprojx$/, /.uvproj$/
          Generators::KeilProjectUpdater
        when /.vcxproj$/
          Generators::MSVSProjectUpdater
        when /\.project$/, /\.cproject$/
          Generators::CubeIDEProjectUpdater
        else
          Kernel.abort "unsupported project file: #{project_file}, #{File.extname(project_file)}"
        end

        generator.new(project_file, touchgfx_path, touchgfx_env, config, @gui_group_name, @generated_group_name).run
      end
    end
  end
end
